/*----------------------- Rogaland University Centre -------------------------
## \section{accur}
## 
## \subsection*{Purpose}
## Compute the classification accuracy of a HIPS classification image.
## 
## 
## \subsection*{Description}
## Accepts two files as input, the correct classification and the 
## classification results. Computes the number of differing pixels.
## 
## 
## \subsection*{Usage}
## \begin{verbatim}
## accur [-perc] imagefile1 [<] imagefile2
## \end{verbatim}
## \begin{description}
##   \item[-perc]
## 	Output percentage, not absolute values
## \end{description}
## 
## 
## 
## \subsection*{Date created}
## April 1993
 *------------------------ Rogaland University Centre -----------------------*/



#include <hipl_format.h>
#include "math.h"


/* Conversion */
int types[] = {PFBYTE,LASTTYPE};

/* Define the commandline-format */
static Flag_Format flagfmt[] = {
  {"perc",{LASTFLAG},0,{
    {PTBOOLEAN,"FALSE"},
    LASTPARAMETER}},
  LASTFLAG};

main(argc,argv)
     int argc;
     char **argv;
{
  struct header hd[2], hdp[2];	/* Nonconverted and conv. input */
  FILE *fp[2];			/* Input HIPS image file pointer */
  Filename filename[2];		/* Input HIPS image file name */
  int method[2];		/* Conversion method */
  byte **img[2];		/* Input/output image referenced as an array */
  int i,j;			/* Window coordinates */
  int fr;			/* Current frame */
  static unsigned class[256][256]; /* Classified pixels according to class */
  Boolean perc;			/* Output percentage, not absolute values */
  byte min_class, max_class;	/* Maximum class number */
  int tot_pix;			/* Total number of pixels in a class */
  int tot_correct;		/* Total number of right classified pixels */
  

  /* Initialize variables and parse commandline */
  hipserrprt = HEL_ERROR;
  Progname = strsave(*argv);
  parseargs(argc,argv,flagfmt,&perc,FFTWO,&(filename[0]),&(filename[1]));

  /* Read the images */
  for (i=0; i<2; i++) {
    fp[i] = hfopenr(filename[i]);
    fread_hdr_a(fp[i],&(hd[i]),filename[i]);
    method[i] = fset_conversion(&(hd[i]),&(hdp[i]),types,filename[i]);
    img[i] = (byte **) matrix2 (&(hdp[i]));
    fread_imagec(fp[i],&(hd[i]),&(hdp[i]),method[i],0,filename[i]);
  }

  /* Check formats etc. */
  if (hd[0].rows != hd[1].rows) 
    perr(HE_C_ROW,filename[1]);
  if (hd[0].cols != hd[1].cols)
    perr(HE_C_COL,filename[1]);
    

  /* Compare the images, pixel by pixel */
  max_class = img[0][0][0];
  min_class = img[0][0][0];
  for(i=0; i<hd[0].rows; i++) {
    for(j=0; j<hd[0].cols; j++) {
      class[img[0][i][j]][img[1][i][j]]++;
      max_class = MAX(max_class,img[0][i][j]);
      min_class = MIN(min_class,img[0][i][j]);
    }
  }
  
  /* Present the results */
  printf("Classified pixels (vert.: class, hor.: classified as..)\n");
  for(j=min_class; j<=max_class; j++) 
    printf("\t%7d",j);
  printf("\n");
  for(i=min_class; i<=max_class; i++) {
    printf("%d:",i);
    if (perc) {
      tot_pix = 0;
      for(j=min_class; j<=max_class; j++)
	tot_pix += class[i][j];
      for(j=min_class; j<=max_class; j++)
	printf("\t%6.2f%%",100. * class[i][j] / tot_pix);
    }
    else 
      for(j=min_class; j<=max_class; j++)
	printf("\t%7d",class[i][j]);
    printf("\n");
  }
  tot_correct = 0;
  for(i=min_class; i<=max_class; i++)
    tot_correct += class[i][i];
  if (perc)
    printf("Overall accuracy %.2f%% correct classified\n",
	   100. * tot_correct / (hd[0].rows * hd[0].cols));
  else
    printf("Overall accuracy %d of %d pixels correct classified\n",
	   tot_correct, hd[0].rows * hd[0].cols);
    
}

