/*----------------------- Rogaland University Centre -------------------------
## \section{analysis}
## 
## \subsection*{Purpose}
## Fullrate analysis tree-structured filter bank.
## 
## \subsection*{Description}
## \subsubsection*{analysis_bank}
## Fullrate analysis tree-structured filter bank.
## 
## \subsection*{Usage}
## \subsubsection*{analysis_bank}
## \begin{verbatim}
## analysis_bank (y,ft,nr,nc)
##      float **y;    
##      FilterTree ft;
##      int nr,nc;    
## \end{verbatim}
## \begin{itemize}
## \item	        Input image
## \item	        Filter structure tree
## \item	        Number of rows
## \item	        Number of columns
## \end{itemize}
## 
## 
## 
## \subsection*{Date created}
## March 1993
 *------------------------ Rogaland University Centre -----------------------*/



/* Header files */
#include "qmf.h"


analysis_bank (y,ft,nr,nc)
     float **y;		/* Input image, pixel pointer */
     FilterTree ft;	/* Filterstructure specification */
     int nr,nc;		/* Number of rows and columns */
     /* Perform two-subband nondecimated filtering in each dimension */
{
  int frame=0;

  analysis_2x2bnd (y,ft,nr,nc,0,getNSubLevels(ft),&frame,FALSE,FALSE);
  return (HIPS_OK);
}





analysis_2x2bnd (y,ft,nr,nc,TrLevel,MaxTrLevel,frame,x_reorg,y_reorg)
     float **y;		/* Input image, pixel pointer */
     FilterTree ft;	/* Filterstructure specification */
     int nr,nc;		/* Number of rows and columns */
     int TrLevel;	/* Filter-tree level */
     int MaxTrLevel;	/* Filter-tree level of the lowest level */
     int *frame;
     Boolean x_reorg, y_reorg;	/* Instruct to reorganize if previous
				   band was a high-pass band */
     /* Perform two-subband filtering in each dimension */
{
  float **x;
  int i, j, m, n, ro, kappa, ro_out, kappa_out;
  struct header hd;
  int noZeroBetwTaps;

  /* Allocating space for one additional image */
  init_hdr_alloc(&hd,"","",1,"",nr,nc,PFFLOAT,1,"");
  x=(float **) matrix2(&hd);

  for (ro=0; ro<2; ro++) {
    for (kappa=0; kappa<2; kappa++) {
      /* Make a copy of the input for further filtering */
      for (m=0;m<nr;m++) 
	for (n=0;n<nc;n++) 
	  x[m][n] = y[m][n];
      
      /* Organizing subbands */
      ro_out=x_reorg ? 1-ro : ro;
      kappa_out=y_reorg ? 1-kappa : kappa;

      filter_2x2bnd(ft->filt_id,x, nr, nc, ro_out, kappa_out,
		    (int) pow (2.0,(double) TrLevel),TRUE);
      h_linscale(&hd,&hd,4.0,0.0);
      
      /* Filtering subbands recursively */
      if (ft->subband[ro_out][kappa_out] != NULL) {
	analysis_2x2bnd (x, ft->subband[ro_out][kappa_out],
			 nr, nc, TrLevel+1, MaxTrLevel, frame,
			 ro!=0,kappa!=0);
      }
      else {
	/* No more subbands, write this subband */
	/* Scale subbands properly */
	h_linscale(&hd,&hd,(float) pow(2.0,(float) MaxTrLevel-TrLevel),0.0);

	/* Bottom level, write the filtered image */
	write_image(&hd,(*frame)++);
      }
    } /* for-kappa */
  } /* for-ro */
  
  /* Freeing space for the additional image */
  free_matrix2(x);
  free_image(&hd);
}

