/*----------------------- Rogaland University Centre -------------------------
## \section{complvq}
## 
## \subsection*{Purpose}
## Compare two LVQ-formatted datafiles, to see what percentage
## they differ by.
## 
## \subsection*{Description}
## Prints number of equal and nonequal lines to the standard error, 
## and all the nonequal lines are printed to the standard output.
## Skips commentlines.
## Assumes that the lines have a maximum length of 4000.
## 
## Warning: this program is not intended to be maximal flexible, but
## be a quick test of two files, for testing/debugging purposes.
## 
## \subsection*{Usage}
## \begin{verbatim}
## complvq file1 file2
## \end{verbatim}
## 
## \subsection*{Date created}
## February 1993
 *------------------------ Rogaland University Centre -----------------------*/



#define MAX_LINE_LEN 4096
#define COMMENT_MARK '#'

#include <stdio.h>
#include <stdlib.h>
#include "mcf.h"


main(argc,argv)
     int argc;
     char **argv;
{
  char *fn1, *fn2;
  FILE *fp1, *fp2;
  char str1[MAX_LINE_LEN], str2[MAX_LINE_LEN];
  unsigned long eq=0, neq=0;

  fn1 = argv[1];
  fn2 = argv[2];

  fp1 = fopen(fn1,"r");
  fp2 = fopen(fn2,"r");

  /* Read vector size */
  while(fgets(str1,MAX_LINE_LEN,fp1)[0] == COMMENT_MARK)
    if (ferror(fp1))
      error();
  while(fgets(str2,MAX_LINE_LEN,fp2)[0] == COMMENT_MARK)
    if (ferror(fp2))
      error();

  /* Print message if vector-sizes are different */
  if(strcmp(str1,str2)!=0)
    fprintf(stderr,"Different vector-sizes\n");

  /* Read lines of the files and compare */
  while(!feof(fp1) && !feof(fp2)) {
    while(fgets(str1,MAX_LINE_LEN,fp1)[0] == COMMENT_MARK)
      if (ferror(fp1))
	error();
    while(fgets(str2,MAX_LINE_LEN,fp2)[0] == COMMENT_MARK)
      if (ferror(fp2))
	error();
    if(!feof(fp1) && !feof(fp2)) 
      if(strcmp(str1,str2)==0) {
	eq++;
      }
      else {
	neq++;
	printf("Line %ld\n",eq+neq);
	printf("  %s",str1);
	printf("  %s\n",str2);
      }
  }
  if((!feof(fp1) && feof(fp2)) || (feof(fp1) && !feof(fp2))) 
    fprintf(stderr,"Files of different lengths\n");
  fprintf(stderr,"Number of equal lines:     \t%ld\n",eq);
  fprintf(stderr,"Number of different lines: \t%ld\n",neq);

  return (0);
}




error()
{
  perror("");
  exit(1);
}
