/*----------------------- Rogaland University Centre -------------------------
## \section{dec\_analysis}
## 
## \subsection*{Purpose}
## Decimated analysis filter bank
## 
## \subsection*{Description}
##
## \subsubsection*{dec\_analysis\_bank}
## 	Apply a decimated analysis filterbank to the input
## 	array/image and write the result to the standard output
## 
## \subsection*{Usage}
##
## \subsubsection*{dec\_analysis\_bank}
## \begin{verbatim}
## dec_analysis_bank(hd,y,ft,frame_no)
##      struct header *hd;
##      float **y;    
##      FilterTree ft;
##      int frame_no;
## \end{verbatim}
## \begin{itemize}
##   \item	Input image header, floating point image
##   \item	Array-reference to the same image with the pixels 
##		accessible
##         	as an array: y[m][n]. See the module matrix2.
##   \item	Filterstructure tree, type FilterTree, see module 
## 		{\em filtertree}.
##   \item	Frame number of input/output frame.
## \end{itemize}
## 
## \subsection*{Date created}
## March 1993
 *------------------------ Rogaland University Centre -----------------------*/



/* Header files */
#include "qmf.h"


dec_analysis_bank(hd,y,ft,frame_no)
     struct header *hd;
     float **y;		/* Array-reference to the image */
     FilterTree ft;	/* Filterstructure specification */
     int frame_no;	/* Input image frame number */
     /* Decimated analysis bank */
{
  /* Filter the input image, recursively */
  dec_analysis_2x2bnd(hd, y, ft, 0, 0, hd->rows, hd->cols, 
		  getNSubLevels(ft),
		  FALSE, FALSE);

  /* Write the utput image (as float) */
  write_image(hd,frame_no);
}



dec_analysis_2x2bnd (hd, y, ft, x_orig, y_orig, x_dim, y_dim,
		 TrLevelDiff, x_reorg, y_reorg)
     struct header *hd;
     float **y;
     FilterTree ft;	/* Filterstructure specification */
     int x_dim, y_dim;
     int x_orig, y_orig;
     int TrLevelDiff;	/* Leveldifference between this level in the
			   filter tree and the bottom level */
     Boolean x_reorg, y_reorg;
     /* Perform two-subband filtering in each dimension */
{
  float **x[2][2];
  float tmp[2][2];
  int i, j, m, n, ro, kappa, ro_out, kappa_out;

  /* Allocating space for 4 nondecimated images */
  for(ro=0;ro<2;ro++)
    for (kappa=0;kappa<2;kappa++)
      x[ro][kappa]=matrix(0,x_dim-1,0,y_dim-1);

  /* Copy the non-filtered image in four copies, one for each LL (Low-Low),
     LH, HL and HH band */
  for (ro=0;ro<2;ro++)
    for (kappa=0;kappa<2;kappa++) 
      for (m=0;m<x_dim;m++) 
	for (n=0;n<y_dim;n++) 
	  x[ro][kappa][m][n] = y[m+x_orig][n+y_orig];

  /* Filtering in each dimension, fullrate */ 
  for (ro=0; ro<2; ro++)
    for(kappa=0; kappa<2; kappa++) 
      filter_2x2bnd(ft->filt_id,x[ro][kappa],x_dim,y_dim,ro,kappa,1,TRUE);

  /* Organizing and subsampling bands */
  for (ro=0;ro<2;ro++) 
    for (kappa=0;kappa<2;kappa++) 
      for (m=0;m<x_dim/2;m++) 
	for (n=0;n<y_dim/2;n++) {
	  ro_out=x_reorg ? 1-ro : ro;
	  kappa_out=y_reorg ? 1-kappa : kappa;
	  y[ro_out*(x_dim/2)+m+x_orig][kappa_out*(y_dim/2)+n+y_orig] = 
	    4*x[ro][kappa][2*m][2*n];
	}

  /* Freeing space for 4 nondecimated images */
  for(ro=0;ro<2;ro++)
    for(kappa=0;kappa<2;kappa++)
      free_matrix(x[ro][kappa],0,x_dim-1,0,y_dim-1);

  /* Filtering subbands recursively */
  for (ro=0; ro<2; ro++) {
    for (kappa=0; kappa<2; kappa++) {
      if (ft->subband[ro][kappa] != NULL) {
	dec_analysis_2x2bnd (hd,y,ft->subband[ro][kappa],
			     x_orig + ro*x_dim/2, y_orig + kappa*y_dim/2,
			     x_dim/2, y_dim/2, TrLevelDiff-1, 
			     ro!=0,kappa!=0);
      }
      else { 
	/* Scale subbands properly */
	setroi(hd, x_orig + ro*x_dim/2, y_orig + kappa*y_dim/2,
	       x_dim/2, y_dim/2);
	h_linscale(hd,hd,(float) pow(2.0,(float) TrLevelDiff),0.0);
	clearroi(hd);
      }
    }
  }
}
