/*----------------------- Rogaland University Centre -------------------------
## \section{even\_gabor}
## 
## 
## \subsection*{Purpose}
## Create the frequency response of an even-symmetric Gabor filter
## 
## \subsection*{Description}
## \subsubsection*{even\_gabor}
## 	Create the frequency response of an even symmetric Gabor filter
## 
## 
## \subsection*{Usage}
## \subsubsection*{even\_gabor}
## \begin{verbatim}
## even_gabor (hd, U, SigmaU, SigmaV, theta)
##   struct header *hd;
##   double U,SigmaU,SigmaV,theta; 
## \end{verbatim}
## \begin{itemize}
## \item	        Image header
## \item	        Centre frequency U of the Gabor function
## 	\item	        $\sigma_U$ of the Gabor function
## 	\item	        $\sigma_V$ of the Gabor function
## 	\item	        Rotation angle, $\theta$
## \end{itemize}
## 
## 
## 
## 
## \subsection*{Date created}
## January 1993
 *------------------------ Rogaland University Centre -----------------------*/


#include <hipl_format.h>
#include	<math.h>
#include "mcf.h"
#define GABOR_NUM_FRAMES 1

/* Create a Fourier-transformed impulse-response of the even-symmetric
   Gabor-function rotated by theta degrees */
even_gabor (hd, U, SigmaU, SigmaV, theta)
     struct header *hd;	/* Output image */
     double U,SigmaU,SigmaV,theta;	/* Filterparameters */
{
  /* Create Fourier-transformed impulse-response */
  return(even_gabor_c(hd,U,SigmaU,SigmaV,theta));
}



even_gabor_c(hd,U,SigmaU,SigmaV,theta)
     struct header *hd;
     double U,SigmaU,SigmaV,theta;
{
  return(even_gabor_C((h_complex *) hd->firstpix,
		     hd->rows,hd->cols,(float) U,
		     (float) SigmaU, (float) SigmaV, (float) theta));
}



/* Low-level routine, performs the actual work */
even_gabor_C(image,nr,nc,U,SigmaU,SigmaV,theta)
     h_complex *image;
     int nr,nc;
     float U,SigmaU,SigmaV,theta;
{
  register int i,j;	/* Coordinates in the output image */
  register float u,v;	/* Corresponding coords. in non-rotated image */
  register float halfInvSigmaUsq,halfInvSigmaVsq,A,realValue;
  register float iCont, jCont;	/* Coordinates: continuous coordinate system */

  theta /= H_180_PI;
  A = 1 / (H_2PI * SigmaU * SigmaV);
  halfInvSigmaUsq =  0.5 / sqr(SigmaU);
  halfInvSigmaVsq =  0.5 / sqr(SigmaV);
  for (i=0;i<nr;i++) {
    iCont = (i<nr/2 ? (float) i / nr : (float) -(nr-i) / nr);
    for (j=0;j<nc;j++) {
      jCont = (j<nc/2 ? (float) j / nc : (float) -(nc-j) / nc);
      u = iCont*cos(theta) + jCont*sin(theta);
      v = jCont*cos(theta) - iCont*sin(theta);

      realValue =
	A*(exp(-sqr(u-U)*halfInvSigmaUsq - sqr(v*nr/nc)*halfInvSigmaVsq)
	   + exp(-sqr(u+U)*halfInvSigmaUsq - sqr(v*nr/nc)*halfInvSigmaVsq)) ;

      (*(image+coord(i,j,nc)))[0] = realValue;
      (*(image+coord(i,j,nc)))[1] = 0;
    }
  }
  return(HIPS_OK);
}









