/*----------------------- Rogaland University Centre -------------------------
## \section{gen\_gabor}
## 
## 
## \subsection*{Purpose}
## Create the frequency response of a general Gabor filter
## 
## \subsection*{Description}
## \subsubsection*{gen\_gabor}
## 	Create the frequency response of a general Gabor filter
## 
## 
## \subsection*{Usage}
## \subsubsection*{gen\_gabor}
## \begin{verbatim}
## gen_gabor (hd, U0, V0, SigmaX, SigmaY, 
#    theta)
##   struct header *hd; 
##   double U0,V0,SigmaX,SigmaY,theta;  
## \end{verbatim}
## \begin{itemize}
##   \item	        Image header
##   \item	        Centre frequency U of the Gabor function
##   \item	        Centre frequency V of the Gabor function
##   \item	        $\sigma_X$ of the Gabor function
##   \item	        $\sigma_Y$ of the Gabor function
##   \item	        Rotation angle $\theta$
## \end{itemize}
## 
## 
## 
## 
## \subsection*{Date created}
## February 1993
 *------------------------ Rogaland University Centre -----------------------*/


/*
  Create an impulseresponse of a Gabor filter as decribed in:
  ``Determining Gabor-filter parameters for texture segmentation'',
  Dennis F. Dunn, William E. Higgins and Joseph Wakeley,
  SPIE Vol. 1826, Intelligent robots and computer vision XI (1992), 51-63
  */

#include <hipl_format.h>
#include <math.h>
#include "mcf.h"
#define GABOR_NUM_FRAMES 1

/* Create a Fourier-transformed impulse-response of the even-symmetric
   Gabor-function rotated by theta degrees */
gen_gabor (hd, U0, V0, SigmaX, SigmaY, theta)
     struct header *hd;	/* Output image */
     double U0,V0,SigmaX,SigmaY,theta;	/* Filterparameters */
{
  /* Create Fourier-transformed impulse-response */
  return(gen_gabor_c(hd,U0,V0,SigmaX,SigmaY,theta));
}



gen_gabor_c(hd,U0,V0,SigmaX,SigmaY,theta)
     struct header *hd;
     double U0,V0,SigmaX,SigmaY,theta;
{
  return(gen_gabor_C((h_complex *) hd->firstpix, hd->rows,hd->cols,
		     (float) U0, (float) V0,
		     (float) SigmaX, (float) SigmaY, (float) theta));
}



/* Low-level routine, performs the actual work */
gen_gabor_C(image,nr,nc,U0,V0,SigmaX,SigmaY,theta)
     h_complex *image;
     int nr,nc;
     float U0,V0,SigmaX,SigmaY,theta;
{
  register int i,j;	/* Coordinates in the output image */
  register float u,v;	/* Corresponding coords. in non-rotated image */
  register float iCont, jCont;	/* Coordinates: continuous coordinate system */

  theta /= H_180_PI;
  for (i=0;i<nr;i++) {
    iCont = (i<nr/2 ? (float) i / nr : (float) -(nr-i) / nr);
    for (j=0;j<nc;j++) {
      jCont = (j<nc/2 ? (float) j / nc : (float) -(nc-j) / nc);
      u = (iCont-U0)*cos(theta) + (jCont-V0)*sin(theta);
      v = (jCont-V0)*cos(theta) - (iCont-U0)*sin(theta);

      (*(image+coord(i,j,nc)))[RE] = 
	exp(-0.5*(sqr(u*SigmaX) + sqr(v*SigmaY)));
      (*(image+coord(i,j,nc)))[IM] = 0;
    }
  }
  return(HIPS_OK);
}









