/*----------------------- Rogaland University Centre -------------------------
## \section{hips2txt}
## 
## 
## \subsection*{Purpose}
## Convert a HIPS-2 image into a ASCII text file, with columns
## separated by spaces and lines separated by carriage returns.
## 
## \subsection*{Usage}
## \begin{verbatim}
## hips2txt [-d decimals<integer>] 
##   [-lowleft] [imagefile]
## \end{verbatim}
## \begin{description}
## \item[-d]
## 	Number of decimals in output, default 0
## \item[-lowleft]
## 	Set origo to the lower left corner
## \end{description}
## 
## 
## 
## 
## 
## \subsection*{Date created}
## February 1993
 *------------------------ Rogaland University Centre -----------------------*/



#include <stdlib.h>
#include <limits.h>
#include <float.h>
#include <hipl_format.h>
#include <math.h>
#include "mcf.h"
#include "error.h"

/* Conversion */
int types[] = {PFFLOAT,LASTTYPE};

/* Define the commandline-format */
static Flag_Format flagfmt[] = {
  {"d",{LASTFLAG},1,{{PTINT,"0","decimals"}, LASTPARAMETER}},
  {"lowleft",{LASTFLAG},0,{{PTBOOLEAN,"FALSE"}, LASTPARAMETER}},
  LASTFLAG};

main(argc,argv)
     int argc;
     char **argv;
{
  struct header hd, hdp;
  FILE *fp;
  Filename filename;
  int i,j;		/* Coordinates in current inputframe */
  int nex;		/* Skipdistance at end of ROI in row to next row */
  register float *pix;	/* Pointer to input picture-element */
  int method;
  int d;		/* Same as parameter -d */
  Boolean lowleft;	/* Same as parameter -lowleft */

  /* Initialize variables and parse commandline */
  hipserrprt = HEL_ERROR;
  Progname = strsave(*argv);
  parseargs(argc,argv,flagfmt,&d,&lowleft,FFONE,&filename);

  fp = hfopenr(filename);
  fread_hdr_a(fp,&hd,filename);
  method = fset_conversion(&hd,&hdp,types,filename);
  nex = hd.ocols - hd.cols;
  fread_imagec(fp,&hd,&hdp,method,0,filename);
  if (lowleft)
    h_rot90(&hdp,&hdp,TRUE);
  pix = (float *) hdp.firstpix;
  for(i=0; i<hd.rows; i++) {
    for(j=0; j<hd.cols; j++) {
      printf("%1.*f ",d,*pix++);
    }
    pix += nex;
    printf("\n");
  }
  
  return(0);
}
