/*----------------------- Rogaland University Centre -------------------------
## \section{matrix2}
## 
## 
## \subsection*{Purpose}
## Make a HIPS-2 image accessible as a 2D matrix
## 
## \subsection*{Description}
## Assigns a column vector with pointes to corresponding lines in
## a HIPS-2 image structure.
## Assigning
## \begin{verbatim}
## y=(float **) matrix2(&hd);
## \end{verbatim}
## will make the pixels of the image associated with hd accessible as
## \begin{verbatim}
## y[i][j]
## \end{verbatim}
## The function automatically detects the data type of the header
## and allocates space according to this.
## 
## \subsubsection*{matrix2}
## Make only the region of interest (see the HIPS-2 documentation for
## setting the region of interest) accessible as the array.
## \subsubsection*{matrix2_all}
## 	Make the whole image available as an array
## 
## 
## \subsection*{Usage}
##
## \subsubsection*{matrix2}
## \begin{verbatim}
## void **
## matrix2 (hd)
##      struct header *hd;
## \end{verbatim}
## \begin{itemize}
## \item		Image header
## \end{itemize}
##
## \subsubsection*{matrix2\_all}
## \begin{verbatim}
## void **
## matrix2_all (hd)
##      struct header *hd;
## \end{verbatim}
## \begin{itemize}
##   \item		Image header
## \end{itemize}
## 
## 
## 
## \subsection*{Date created}
## March 1993
 *------------------------ Rogaland University Centre -----------------------*/


#include <hipl_format.h>
#include <malloc.h>


void **
matrix2 (hd)
     struct header *hd;
{
  void **matrixPtr, *pixPtr;
  int i, elem_size;

  elem_size = sizeof_pix (hd);
  matrixPtr = (void **) malloc((unsigned) hd->rows * sizeof(void *));
  pixPtr = (void *) hd->firstpix;
  for (i=0; i<hd->rows; i++) {
    matrixPtr[i] = (void *)pixPtr;
    pixPtr += elem_size * hd->ocols;
  }
  return (matrixPtr);
}



void **
matrix2_all (hd)
     struct header *hd;
{
  void **matrixPtr, *pixPtr;
  int i, elem_size;

  elem_size = sizeof_pix(hd);
  matrixPtr = (void **) malloc((unsigned) hd->orows * elem_size);
  pixPtr = (void *) hd->image;
  for (i=0; i<hd->orows; i++) {
    matrixPtr[i] = (void *)pixPtr;
    pixPtr += elem_size * hd->ocols;
  }
  return (matrixPtr);
}



free_matrix2 (ptr)
     void **ptr;
{
  free(ptr);
}



free_matrix2_all (ptr)
     void **ptr;
{
  return (free_matrix2(ptr));
}



sizeof_pix(hd)
     struct header *hd;
{
  switch (hd->pixel_format) {
    case PFBYTE:	return (sizeof(byte));
    case PFSHORT:	return (sizeof(short));
    case PFINT:		return (sizeof(int));
    case PFFLOAT:	return (sizeof(float));
    case PFCOMPLEX:	return (sizeof(h_complex));
    case PFDOUBLE:	return (sizeof(double));
    case PFDBLCOM:	return (sizeof(h_dblcom));
    case PFSBYTE:	return (sizeof(sbyte));
    case PFUSHORT:	return (sizeof(h_ushort));
    case PFUINT:	return (sizeof(h_uint));
    default:		perr(HE_FMT,"sizeof_pix...");
  }
}


