/*----------------------- Rogaland University Centre -------------------------
## \section{mcf\_gauss}
## 
## 
## \subsection*{Purpose}
## Gaussian lowpass filter.
## 
## \subsection*{Description}
## \subsubsection*{mcf\_gauss}
## 	Applies a gaussian lowpass filter to the input.
## 	The filtering is performed in the Fourier domain, and
## 	this function therefore applies both forward and reverse FFT
## 
## \subsection*{Usage}
## \subsubsection*{mcf\_gauss}
## \begin{verbatim}
## mcf_gauss(hdi,hdo,sigma)
##   struct header *hdi, *hdo;
##   double sigma;
## \end{verbatim}
## \begin{itemize}
##   \item	        Input image (complex)
##   \item	        Output image (complex)
##   \item	        $\sigma$, parameter to the Gaussian
## \end{itemize}
## 
## 
## 
## 
## \subsection*{Date created}
## January 1993
 *------------------------ Rogaland University Centre -----------------------*/

#include <hipl_format.h>
#include <math.h>
#include "mcf.h"


mcf_gauss(hdi,hdo,sigma)
     struct header *hdi, *hdo;
     double sigma;
{
  if(hdi->pixel_format==PFCOMPLEX && hdo->pixel_format==PFCOMPLEX) {
    return(mcf_gauss_c(hdi,hdo,sigma));
  }
  else {
    perr(HE_FMTSUBR,"mcf_gauss (input)",hformatname(hdi->pixel_format));
    perr(HE_FMTSUBR,"mcf_gauss (output)",hformatname(hdo->pixel_format));
    return(HIPS_ERROR);
  }
} 
 

mcf_gauss_c(hd,sigma)
     struct header *hd;
     double sigma;
{
  h_fourtr(hd);
  mcf_gauss_C((h_complex *) hd->firstpix,
	      hd->rows,hd->cols,hd->ocols,(float) sigma);
  return(mcf_invfourtr(hd));
}


mcf_gauss_C(image,nr,nc,nlp,sigma)
     h_complex *image;
     int nr, nc, nlp;
     float sigma;
{
  register int i,j;     /* Coordinates */
  register float halfInvSigmaSq;
  register float u, v;  /* Coordinates: continuous coordinate system */
  register float gauss;	/* Gaussian */
  register h_complex *pix;
  int nex;

  pix = image;
  nex = nlp-nc;
  /* Convert sigma from time to frequency domain */
  sigma = 1 / (H_2PI * sigma);
  halfInvSigmaSq =  0.5 / sqr(sigma);
  for (i=0;i<nr;i++) {
    u = (i<nr/2 ? (float) i / nr : (float) -(nr-i) / nr);
    for (j=0;j<nc;j++) {
      v = (j<nc/2 ? (float) j / nc : (float) -(nc-j) / nc);
      gauss =
        (exp(-(sqr(u) + sqr(v))*halfInvSigmaSq));
      (*pix)[RE] = (*pix)[RE] * gauss;
      (*pix)[IM] = (*pix)[IM] * gauss;
      pix++;

    }
    pix += nex;
  }
  return(HIPS_OK);
}
