/*----------------------- Rogaland University Centre -------------------------
## \section{mcf\_tanh\_nonlin}
## 
## 
## \subsection*{Purpose}
## Apply the tanh(..) nonlinearity described in F. Farrokhnia's 
## PhD dissertation to an image
## 
## \subsection*{Description}
## \subsubsection*{mcf\_tanh\_nonlin}
## 	Function applying the tanh(..) nonliarity
## 
## \subsection*{Usage}
## \subsubsection*{mcf\_tanh\_nonlin}
## \begin{verbatim}
## mcf_tanh_nonlin(hdi,hdo,alpha)
##   struct header *hdi, *hdo;
##   double alpha;
## \end{verbatim}
## \begin{itemize}
##   \item	        Input image header.
##   \item	        Output image header.
##   \item	        alpha: parameter to the nonlinear transform.
## \end{itemize}
## 
## 
## 
## 
## 
## \subsection*{Date created}
## January 1993
 *------------------------ Rogaland University Centre -----------------------*/


#include <hipl_format.h>
#include <math.h>
#include "mcf.h"

mcf_tanh_nonlin(hdi,hdo,alpha)
     struct header *hdi, *hdo;
     double alpha;
{
  if(hdi->pixel_format==PFCOMPLEX && hdo->pixel_format==PFCOMPLEX) {
    /* Nonlinear transform */
    return(mcf_tanh_nonlin_c(hdi,hdo,alpha));
  }
  else if(hdi->pixel_format==PFFLOAT && hdo->pixel_format==PFFLOAT) {
    /* Nonlinear transform */
    return(mcf_tanh_nonlin_f(hdi,hdo,alpha));
  }
  else {
    perr(HE_FMTSUBR,"mcf_tanh_nonlin (input)",hformatname(hdi->pixel_format));
    perr(HE_FMTSUBR,"mcf_tanh_nonlin (output)",hformatname(hdo->pixel_format));
    return(HIPS_ERROR);
  }
}



mcf_tanh_nonlin_c(hdi,hdo,alpha)
     struct header *hdi,*hdo;
     double alpha;
{
  return (mcf_tanh_nonlin_C((h_complex *) hdi->firstpix,
			    (h_complex *) hdo->firstpix,
			    hdi->rows,hdi->cols,hdi->ocols,hdo->ocols,alpha));
}


/* Low-level routine, perform the actual work */
mcf_tanh_nonlin_C(imagei,imageo,nr,nc,nlpi,nlpo,alpha)
     h_complex *imagei, *imageo;
     int nr,nc,nlpi,nlpo;
     double alpha;
{
  register int y,x;		/* Coordinates */
  
  /* Traverse each pixel in the outputimage */
  for (x=0;x<nr;x++) {
    for (y=0;y<nc;y++) {
      (*(imageo+coord(x,y,nlpo)))[0]  
	= tanh(alpha * 
	       sqrt(sqr((*(imagei+coord(x,y,nlpi)))[0])
		    + sqr((*(imagei+coord(x,y,nlpi)))[1])));
      (*(imageo+coord(x,y,nlpo)))[1] = 0;
    }
  }
  return(HIPS_OK);
}




mcf_tanh_nonlin_f(hdi,hdo,alpha)
     struct header *hdi,*hdo;
     double alpha;
{
  return (mcf_tanh_nonlin_F((float *) hdi->firstpix,
			    (float *) hdo->firstpix,
			    hdi->rows,hdi->cols,hdi->ocols,hdo->ocols,alpha));
}


/* Low-level routine, perform the actual work */
mcf_tanh_nonlin_F(imagei,imageo,nr,nc,nlpi,nlpo,alpha)
     float *imagei, *imageo;
     int nr,nc,nlpi,nlpo;
     double alpha;
{
  register int y,x;		/* Coordinates */
  
  /* Traverse each pixel in the outputimage */
  for (x=0;x<nr;x++) {
    for (y=0;y<nc;y++) {
      (*(imageo+coord(x,y,nlpo)))
	= tanh(alpha * 
	       sqrt(sqr((*(imagei+coord(x,y,nlpi))))
		    + sqr((*(imagei+coord(x,y,nlpi))))));
    }
  }
  return(HIPS_OK);
}




