/*----------------------- Rogaland University Centre -------------------------
/*---------------------------------------------------------------------------
## \section{mindec}
## 
## \subsection*{Purpose}
## Return the least common decimation factor for a filter structure
## 
## \subsection*{Usage }
## \begin{verbatim}
## mindec 
##    [-sn struct<integer> |  
##     -s struct<string> |  
##     -sb struct<string>] 
## \end{verbatim}
## \begin{description}
##   \item[-s]
##      Filter structure string, defined as a quad tree.
##      See the module {\em filtertree} (CompileFiltStructTree)
##      for a description of the structure.
##   \item[-sb]
##      Filterstructure string, defined as a binary tree.
##      See the module {\em filtertree} (CompileFiltStructTreeFromBin)
##      for a description of the structure.
##   \item[-sn]
##      Filter structure number, between 1 and 11 (default 4).
##      See the module {\em filtertree} (BuildFiltStructTree)
##      for a description of the structure.
## \end{description}
## 
## \subsection*{Date created}
## 4/3-94
+---------------------------------------------------------------------------+
|	Trygve Randen                                                       |
|	Rogaland University Center, Dept. of Elect. and Computer Eng.       |
|	P.O. Box 2557 Ullandhaug, 4004 Stavanger, Norway                    |
|	Tel.: +47 51 87 43 36, Fax.: +47 51 87 42 36                        |
|	E-mail: tranden@hsr.no                                              |
+--------------------------------------------------------------------------*/


#include <hipl_format.h>
#include <math.h>
#include "qmf.h"
#include "math.h"

/* Define the commandline-format */
static Flag_Format flagfmt[] = {
  {"sn",{"s","sb",LASTFLAG},1,{
    {PTBOOLEAN,"TRUE"},
    {PTINT,"4","struct"},
    LASTPARAMETER}},
  {"sb",{"sn","s",LASTFLAG},1,{
    {PTBOOLEAN,"FALSE"},
    {PTSTRING,"(,)","struct"},
    LASTPARAMETER}},
  {"s",{"sn","sb",LASTFLAG},1,{
    {PTBOOLEAN,"FALSE"},
    {PTSTRING,"(,)","struct"},
    LASTPARAMETER}},
  LASTFLAG};







main(argc,argv)
     int argc;
     char **argv;
{
  Boolean structNoFlag;		/* Structure number specified */
  Boolean structStr1DFlag;	/* Structure string specified (bin) */
  Boolean structStrFlag;	/* Structure string specified (quad) */
  int structNo;			/* Structure description number  */
  char *structStr1D;		/* Structure description string (bin) */
  char *structStr;		/* Structure description string (quad) */
  FilterTree filtertree;	/* Filter structure tree */

  Progname = strsave(*argv);
  parseargs(argc,argv, flagfmt, &structNoFlag, &structNo, 
	    &structStr1DFlag, &structStr1D, &structStrFlag, &structStr,	    
	    FFNONE);

  /* Build the filter structure binary tree */
  if (structStrFlag) 
    filtertree = CompileFiltStructTree(structStr,"");
  else if (structStr1DFlag) 
    filtertree = CompileFiltStructTreeFromBin(structStr1D,"");
  else if (structNoFlag)
    filtertree = BuildFiltStructTree(structNo,"");

  printf("%d\n",(int) pow(2.0,(float) getUpperSubLevel (filtertree)+1));

  return(0);
}




