/*----------------------- Rogaland University Centre -------------------------
## \section{oddeven}
## 
## \subsection*{Purpose}
## Program for generating an odd or an even texture, see:
## 	``Visual discrimination of textures with identical third-order 
## 	statistics'',
## 	B. Julesz, E. N. Gilbert and J. D. Victors,
## 	Biological Cybernetics 31, pp 137-140, 1978
## 
## 
## \subsection*{Description}
## Generates either an `odd' texture (a texture where the 2x2 pixel blocks
## have an odd number of black pixels) or an `even' texture (a texture where 
## the 2x2 pixel blocks have an odd number of black pixels).
## 
## \subsection*{Usage}
## \begin{verbatim}
## oddeven [-s rows<integer> 
##             [cols<integer>]] 
##    [-odd | -even]
## \end{verbatim}
## \begin{description}
## \item[-s]
## 	rows [columns]
## 	Image size
## \item[-odd]
## 	Generate an `odd' image
## \item[-even]
## 	Generate an `even' image
## \end{description}
## 
## 
## 
## \subsection*{Date created}
## January 1993
 *------------------------ Rogaland University Centre -----------------------*/


#include <stdlib.h>
#include <limits.h>
#include <float.h>
#include <hipl_format.h>
#include <math.h>
#include "mcf.h"

/* Return rank of the first number in the argument, i.e. if a is biggest,
   return 1, second biggest: 2, ... */
oe_rank(a,b,c,d) 
     int a, b, c, d;
{
  if(a>b)
    if(a>c)
      if(a>d) return (1);
      else return (2);
    else 
      if(a>d) return (2);
      else return(3);
  else
    if(a>c)
      if(a>d) return (2);
      else return (3);
    else 
      if(a>d) return (3);
      else return(4);      
}


/* Define the commandline-format */
static Flag_Format flagfmt[] = {
  {"s",{LASTFLAG},1,{{PTINT,"128","rows"},{PTINT,"-1","cols"},LASTPARAMETER}},
  {"odd",{"even",LASTFLAG},0,{{PTBOOLEAN,"TRUE"}, LASTPARAMETER}},
  {"even",{"odd",LASTFLAG},0,{{PTBOOLEAN,"FALSE"}, LASTPARAMETER}},
  LASTFLAG};

main(argc,argv)
     int argc;
     char **argv;
{
  struct header hd;
  int rows,cols,d[4],i,j;
  double num_black;
  Boolean odd,even;
  byte *pix;
  char *seq_name;

  Progname = strsave(*argv);
  parseargs(argc,argv,flagfmt,&rows,&cols,&odd,&even,FFNONE);
  if(cols<0)
    cols = rows;
  if((rows%2!=0) || (cols%2!=0)) {
    perr(HE_MSG,"Number of rows and number of columns must be divisible by 2");
    exit(1);
  }
  srand(time(NULL));

  if(even)
    seq_name = "Even: every 2x2 cell has an even number of black pixels";
  else /* odd */
    seq_name = "Odd: every 2x2 cell has an odd number of black pixels";
  init_hdr_alloc(&hd,"","",1,"",rows,cols,PFBYTE,1,"");
  pix = (byte *) hd.firstpix;
  for(i=0; i<rows; i += 2) {
    for(j=0; j<cols; j += 2) {

      /* Determine number of black pixels in current 2x2 cell */
      num_black = ((float) rand()) / ((float) RAND_MAX);
      if(even)
	num_black = (float)(2*((int) (num_black*2+0.5)));
      else /* odd */
	num_black = (float)(2*((int) (num_black+0.5))+1);

      /* Distribute the black pixels randomly in the cell */
      d[0] = rand(); d[1] = rand(); d[2] = rand(); d[3] = rand();
      *(pix+coord(i  ,j  ,cols))
	=(oe_rank(d[0],d[1],d[2],d[3])>(int) num_black ? 0:255);
      *(pix+coord(i  ,j+1,cols)) 
	=(oe_rank(d[1],d[0],d[2],d[3])>(int) num_black ? 0:255);
      *(pix+coord(i+1,j  ,cols))
	=(oe_rank(d[2],d[0],d[1],d[3])>(int) num_black ? 0:255);
      *(pix+coord(i+1,j+1,cols))
	=(oe_rank(d[3],d[0],d[1],d[2])>(int) num_black ? 0:255);
    }
  }
  write_headeru(&hd,argc,argv);
  write_image(&hd,0);
  return(0);
}



