/*----------------------- Rogaland University Centre -------------------------
## \section{raw2hips}
## 
## 
## \subsection*{Purpose}
## Program for converting a raw datafile to HIPS-2 format.
## 
## \subsection*{Description}
## Takes number of rows and columns from the commandline and converts the image.
## The output is written to the standard output.
## 
## Restriction: does only accept byte-format input.
## 
## \subsection*{Usage}
## \begin{verbatim}
## raw2hips [-s rows<integer> 
##   [cols<integer>]]
##   [-byte | -short | -int | -float | 
##    -complex | -double | -sbyte | 
##    -ushort | -uint]
## \end{verbatim}
## \begin{description}
##   \item[-s]
## 	Size of the input image, default 128 rows and the same 
## 	number of columns as rows.
##   \item[-byte]
## 	Read a byte formatted raw file
##   \item[-short]
## 	Read a short formatted raw file
##   \item[-int]
## 	Read a int formatted raw file
##   \item[-float]
## 	Read a float formatted raw file
##   \item[-complex]
## 	Read a complex formatted raw file
##   \item[-double]
## 	Read a double formatted raw file
##   \item[-sbyte]
## 	Read a sbyte formatted raw file
##   \item[-ushort]
## 	Read a ushort formatted raw file
##   \item[-uint]
## 	Read a uint formatted raw file
## \end{description}
## 
## 
## 
## 
## \subsection*{Date created}
## February 1993
 *------------------------ Rogaland University Centre -----------------------*/


#include <hipl_format.h>

/* Define the commandline-format */
static Flag_Format flagfmt[] = {
  {"s",{LASTFLAG},1,{
    {PTINT,"128","rows"},
    {PTINT,"-1","cols"},
    LASTPARAMETER}},
  {"byte",{"short","int","float","complex","double","sbyte","ushort","uint",LASTFLAG},0,{
    {PTBOOLEAN,"TRUE"},
    LASTPARAMETER}},
  {"short",{"byte","int","float","complex","double","sbyte","ushort","uint",LASTFLAG},0,{
    {PTBOOLEAN,"FALSE"},
    LASTPARAMETER}},
  {"int",{"byte","short","float","complex","double","sbyte","ushort","uint",LASTFLAG},0,{
    {PTBOOLEAN,"FALSE"},
    LASTPARAMETER}},
  {"float",{"byte","short","int","complex","double","sbyte","ushort","uint",LASTFLAG},0,{
    {PTBOOLEAN,"FALSE"},
    LASTPARAMETER}},
  {"complex",{"byte","short","int","float","double","sbyte","ushort","uint",LASTFLAG},0,{
    {PTBOOLEAN,"FALSE"},
    LASTPARAMETER}},
  {"double",{"byte","short","int","float","complex","sbyte","ushort","uint",LASTFLAG},0,{
    {PTBOOLEAN,"FALSE"},
    LASTPARAMETER}},
  {"sbyte",{"byte","short","int","float","complex","double","ushort","uint",LASTFLAG},0,{
    {PTBOOLEAN,"FALSE"},
    LASTPARAMETER}},
  {"ushort",{"byte","short","int","float","complex","double","sbyte","uint",LASTFLAG},0,{
    {PTBOOLEAN,"FALSE"},
    LASTPARAMETER}},
  {"uint",{"byte","short","int","float","complex","double","sbyte","ushort",LASTFLAG},0,{
    {PTBOOLEAN,"FALSE"},
    LASTPARAMETER}},
  LASTFLAG};

main(argc,argv)
     int argc;
     char **argv;
{
  struct header hd;
  int rows,cols,i,j;
  int format, size;
  Boolean pfbyte,pfshort,pfint,pffloat,pfcomplex,pfdouble,pfsbyte,pfushort,pfuint;

  Progname = strsave(*argv);
  parseargs(argc,argv,flagfmt,&rows,&cols,&pfbyte,&pfshort,&pfint,&pffloat,
	    &pfcomplex,&pfdouble,&pfsbyte,&pfushort,&pfuint,
	    FFNONE);
  if(cols<0)
    cols = rows;

  if (pfbyte) {
    format=PFBYTE;
    size=sizeof(unsigned char);
  }
  else if (pfshort) {
    format=PFSHORT;
    size=sizeof(short);
  }
  else if (pfint) {
    format=PFINT;
    size=sizeof(int);
  }
  else if (pffloat) {
    format=PFFLOAT;
    size=sizeof(float);
  }   
  else if (pfcomplex) {
    format=PFCOMPLEX;
     size=2*sizeof(float);
  }
  else if (pfdouble) {
    format=PFDOUBLE;
     size=sizeof(double);
  }
  else if (pfsbyte) {
    format=PFSBYTE;
     size=sizeof(char);
  }
  else if (pfushort) {
    format=PFUSHORT;
     size=sizeof(unsigned short);
  }
  else if (pfuint) {
    format=PFUINT;
     size=sizeof(unsigned int);
  }

  init_hdr_alloc(&hd,"","",1,"",rows,cols,format,1,"");
  fread(hd.firstpix,size,rows*cols,stdin);
  write_headeru(&hd,argc,argv);
  write_image(&hd,0);
  return(0);
}

