/*----------------------- Rogaland University Centre -------------------------
## \section{snr}
## 
## 
## \subsection*{Purpose}
## Compute the signal to noise ratio between two images
## 
## \subsection*{Description}
## Same program as snr written by John Haakon Husoey, but adapted
## to HIPS version 2.
## 
## \subsection*{Usage}
## \begin{verbatim}
## snr imagefile1 [<] imagefile2
## \end{verbatim}
 *------------------------ Rogaland University Centre -----------------------*/



#include <stdio.h>
#include <math.h>
#include <hipl_format.h>
#include <sys/file.h>

struct header hd1,hd2,hdp1,hdp2;
unsigned char **pict1_uch, **pict2_uch;
float **pict1_flt, **pict2_flt;
double **pict1_dbl, **pict2_dbl;

static Flag_Format flagfmt[]={
  LASTFLAG};

int types[]={PFBYTE,PFFLOAT,PFDOUBLE,LASTTYPE};

main(argc,argv)
     int argc;
     char **argv;
{
  int i,j,m,n;
  float snr_uch();
  float snr_flt();
  float snr_dbl();
  FILE *f1,*f2;
  Filename filename1, filename2;
  int method1, method2;

  /* Initialize variables */
  Progname=strsave(*argv);
  parseargs(argc,argv,flagfmt,FFTWO,&filename1,&filename2);

  f1 = hfopenr(filename1);
  f2 = hfopenr(filename2);
  fread_hdr_a(f1,&hd1,filename1);
  fread_hdr_a(f2,&hd2,filename2);
  method1 = fset_conversion(&hd1,&hdp1,types,filename2);
  method2 = fset_conversion(&hd2,&hdp2,types,filename2);

  if ((    (hdp1.pixel_format == hdp2.pixel_format)
       &&  (hdp1.cols == hdp2.cols)
       &&  (hdp1.rows == hdp2.rows)
       ) == 0) {
    fprintf(stderr,"Incompatible pictures");
    exit(1);
  }

  if (hdp1.num_frame > hdp2.num_frame) {
    hdp1.num_frame = hdp2.num_frame;
  }

  switch(hdp1.pixel_format)
    {
    case PFBYTE: 
      for (j=0;j<hdp1.num_frame;j++) {
	fread_imagec(f1,&hd1,&hdp1,method1,j,filename1);
	fread_imagec(f2,&hd2,&hdp2,method2,j,filename2);
	pict1_uch=(byte **) matrix2(&hdp1);
	pict2_uch=(byte **) matrix2(&hdp2);
	fprintf(stdout," %d   %f\n", j, snr_uch());
	free_matrix2(pict1_uch);
	free_matrix2(pict2_uch);
      }
      break;
    case PFFLOAT:
      for (j=0;j<hdp1.num_frame;j++) {
	fread_imagec(f1,&hd1,&hdp1,method1,j,filename1);
	fread_imagec(f2,&hd2,&hdp2,method2,j,filename2);
	pict1_flt=(float **) matrix2(&hdp1);
	pict2_flt=(float **) matrix2(&hdp2);
	fprintf(stdout," %d   %f\n", j, snr_flt());
	free_matrix2(pict1_flt);
	free_matrix2(pict2_flt);
      }
      break;
    case PFDOUBLE:
      for (j=0;j<hdp1.num_frame;j++) {
	pict1_dbl=(double **) matrix2(&hdp1);
	pict2_dbl=(double **) matrix2(&hdp2);
	fread_imagec(f1,&hd1,&hdp1,method1,j,filename1);
	fread_imagec(f2,&hd2,&hdp2,method2,j,filename2);
	fprintf(stdout," %d   %f\n", j, snr_dbl());
	free_matrix2(pict1_dbl);
	free_matrix2(pict2_dbl);
      }
      break;
    default: fprintf(stderr,"only BYTE, FLOAT and DOUBLE\n");
      exit(1);
      break; 
    }
}



float snr_uch()
{
  int i,j;
  float denom,sum,tmp;
  
  sum=0;
  for (i=0;i<hdp1.rows;i++) {
    for (j=0;j<hdp1.cols;j++) {
      tmp=(float)pict2_uch[i][j]-(float)pict1_uch[i][j];
      sum=sum+(tmp*tmp);
    };
  };
  denom=hdp1.rows*hdp1.cols*255.0*255.0;
  return(10*log10(denom/sum));
};

float snr_flt()
{
  int i,j;
  float denom,sum,tmp;
  
  sum=0;
  for (i=0;i<hdp1.rows;i++) {
    for (j=0;j<hdp1.cols;j++) {
      tmp=pict2_flt[i][j]-pict1_flt[i][j];
      sum=sum+(tmp*tmp);
    };
  };
  denom=hdp1.rows*hdp1.cols*255.0*255.0;
  return(10*log10(denom/sum));
};


float snr_dbl()
{
  int i,j;
  float denom,sum,tmp;
  
  sum=0;
  for (i=0;i<hdp1.rows;i++) {
    for (j=0;j<hdp1.cols;j++) {
      tmp=(float)pict2_dbl[i][j]-(float)pict1_dbl[i][j];
      sum=sum+(tmp*tmp);
    };
  };
  denom=hdp1.rows*hdp1.cols*255.0*255.0;
  return(10*log10(denom/sum));
};


