/*---------------------------------------------------------------------------
## \section{statsnorm}
## 
## \subsection*{Purpose}
## Normalize an image according to its statistics, i.e. set the mean
## to zero and the variance to one.
## 
## \subsection*{Description}
## The output will be float.
## 
## \subsection*{Usage }
## \begin{verbatim}
## statsnorm [imagefile]
## \end{verbatim}
## 
## \subsection*{Data formats handled directly}
## \begin{itemize}
##    \item	float
## \end{itemize}
##
## \subsection*{Date created}
## June 1993
+---------------------------------------------------------------------------+
|	Trygve Randen                                                       |
|	Rogaland University Center, Dept. of Elect. and Computer Eng.       |
|	P.O. Box 2557 Ullandhaug, 4004 Stavanger, Norway                    |
|	Tel.: +47 51 87 43 36, Fax.: +47 51 87 42 36                        |
|	E-mail: tranden@hsr.no                                              |
+--------------------------------------------------------------------------*/


#include <hipl_format.h>


/* Conversion */
int types[] = {PFFLOAT,LASTTYPE};

/* Define the commandline-format */
static Flag_Format flagfmt[] = {
  LASTFLAG};

main(argc,argv)
     int argc;
     char **argv;
{
  struct header hd, hdp;	/* Nonconverted and conv. in, conv. out. */
  FILE *fp;			/* Input HIPS image file pointer */
  Filename filename;		/* Input HIPS image file name */
  int fr;			/* Current frame */
  int method;			/* Conversion method */
  struct hips_stats stats;	/* Frame statistics */

  /* Initialize variables and parse commandline */
  hipserrprt = HEL_ERROR;
  Progname = strsave(*argv);
  parseargs(argc,argv,flagfmt,FFONE,&filename);

  /* Open file and initiate image */
  fp = hfopenr(filename);
  fread_hdr_a(fp,&hd,filename);
  method = fset_conversion(&hd,&hdp,types,filename);
  write_headeru(&hdp,argc,argv);

  /* Normalize statistics for each frame */
  for (fr=0; fr<hd.num_frame; fr++) {

    /* Read input */
    fread_imagec(fp,&hd,&hdp,method,0,filename);

    /* Compute statistics */
    h_stats(&hdp,&stats,FALSE);

    /* Normalize statistics */
    h_linscale(&hdp,&hdp,1/stats.stdev,-stats.mean/stats.stdev);

    /* Write normalized image */
    write_imagec(&hd,&hdp,method,FALSE,fr);
  }
}
