/*----------------------- Rogaland University Centre -------------------------
 *
 *
 * module:
 *	header	struct_defs
 *
 *
 * purpose:
 *	Definitions of structures for analysis and synthesis FIR and IIR
 *	filterbanks
 *
 * description:
 *
 * parameters:
 *
 * routines required:
 *
 * author / maintained by:
 *	Trygve Randen
 *
 * date created:
 *	March 22, 1993
 *
 * updates:
 *
 *
 *------------------------ Rogaland University Centre -----------------------*/

#include "qmf.h"

#ifndef _STRUCT_DEFS_H_
#define _STRUCT_DEFS_H_

#define MAX_FILT_FILE_NAME 200
#define MAX_FILT_ID 80

/* IIR filterspec */
struct filterspec {
  char filterfile[MAX_FILT_FILE_NAME];
  int no_of_branches;
  int filter_order[8];		/* max 8 branches */
  float a[8][MAX_NTAPS];	/* max 8 branches and 4 coefficients
				   (i.e. 4 1 st order sections in cascade)
				   */
};


/* FIR polyphase qmf specification */
struct fir_filterspec { 
  char filterfile[MAX_FILT_FILE_NAME];
  int no_of_taps;
  /* polyphase filters for a maximum of 16 taps i.e. the maximum
     size of the corresponding polyphase filter is 32.
     */
  float p0[MAX_NTAPS];
  float p1[MAX_NTAPS];
};


/* FIR specification */
struct firspec {
  char filterfile[MAX_FILT_FILE_NAME];
  int no_of_taps;
  int channels;
  
  /* Filters for a maximum of MAX_NTAPS taps and 8 channels */
  float a[8][MAX_NTAPS];
};


/* Binary filter-tree structure definition. Each node in the tree implies one
   two-band filter. L indicateslow-band split, H high band split */
typedef struct FiltTreeStruct1D * FilterTree1D;
struct FiltTreeStruct1D {
  FilterTree1D subband[2];
  char filt_id[MAX_FILT_ID];
};


/* Quad filter-tree structure definition. Each node in the tree implies one
   2x2-band filter. L=low band, H=high band => LL low-low band ... */
typedef struct FiltTreeStruct2D * FilterTree2D;
typedef struct FiltTreeStruct2D * FilterTree;
struct FiltTreeStruct2D {
  FilterTree subband[2][2];
  char filt_id[MAX_FILT_ID];
};


#endif
