/*----------------------- Rogaland University Centre -------------------------
## \section{win\_fourtr}
## 
## 
## \subsection*{Purpose}
## Code for computing the windowed Fourier-transform of an image.
## 
## \subsection*{Description}
## \subsubsection*{win\_fourtr}
## 	Compute the windowed Fourier-transform of an image.
## Assumes that the input image is complex ({\tt h_complex}) formatted.
## 
## \subsection*{Usage}
## \subsubsection*{win\_fourtr}
## \begin{verbatim}
## win_fourtr(hdi,hdo,s,sc,SigmaX,SigmaY)
##      struct header *hdi, *hdo;
##      Coord *s;
##      int sc;
##      double SigmaX, SigmaY;
## \end{verbatim}
## \begin{itemize}
##   \item	        Input image.
##   \item	        Output image.
##   \item	        List of window centre corrdinates.
##   \item	        Index in list of window centre corrdinates.
##   \item	        Sigma X: parameter to the Gaussian window.
##   \item	        Sigma Y.
## \end{itemize}
## 
## \subsection*{Date created}
## March 1993
 *------------------------ Rogaland University Centre -----------------------*/
  
#include <stdlib.h>
#include <limits.h>
#include <float.h>
#include <hipl_format.h>
#include <math.h>
#include "mcf.h"
#include "math.h"




double
g(x,y,SigmaX,SigmaY)
     int x,y;
     double SigmaX, SigmaY;
     /* Gaussian window function */
{
  /* NB! using discrete coordinates. This has implication on SigmaX
     and SigmaY. 
     */
  return (1 / (H_2PI * SigmaX * SigmaY) *
          exp(-0.5 * (sqr(x/SigmaX) + sqr(y/SigmaY))));
}




win_fourtr(hdi,hdo,s,sc,SigmaX,SigmaY)
     struct header *hdi, *hdo;
     Coord *s;
     int sc;
     double SigmaX, SigmaY;
     /* Compute the windowed Fourier-transform, Gaussian window */
{
  int i, j;
  float *ipix;
  h_complex *opix;
  int nexi=hdi->orows - hdi->rows;
  int nexo=hdo->orows - hdo->rows;
  struct header hd_c;
  float div = (float) hdi->rows * hdi->cols;

  init_hdr_alloc(&hd_c,"","",1,"",hdi->orows,hdi->ocols, PFCOMPLEX,1,"");
  ipix = (float *) hdi->firstpix;
  opix = (h_complex *) hd_c.firstpix;
  /* Calculate the window function */
  for(i=0; i<hdi->rows; i++) {
    for(j=0; j<hdi->cols; j++) {
      (*opix)[RE] = *ipix * g(i-(*(s+sc))[0], j-(*(s+sc))[1], SigmaX, SigmaY);
      (*opix)[IM] = 0;
      opix++;
      ipix++;
    }
    opix += nexo;
    ipix += nexi;
  }
  h_fourtr(&hd_c);
  h_tof(&hd_c,hdo);
  h_divscale(hdo,hdo,&div);
  free_image(&hd_c);

  return(HIPS_OK);
}



